﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;

using MakeWindowFullscreen.WinApi;

namespace MakeWindowFullscreen
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        private void Application_Startup(object sender, StartupEventArgs e)
        {
            var args = Environment.GetCommandLineArgs();

            if (args != null && args.Length > 1)
            {
                string targetProcessName = null;

                int left, top, width, height;

                if (args.Length > 1)
                {
                    targetProcessName = args[1];
                    if (string.IsNullOrEmpty(targetProcessName))
                    {
                        Console.Error.WriteLine("Expected a process name.");
                        this.Shutdown(1);
                        return;
                    }
                }

                if (args.Length > 2)
                {
                    if (!int.TryParse(args[2], out left))
                    {
                        Console.Error.WriteLine("Failed to parse the left parameter: {0}", args[2]);
                        this.Shutdown(2);
                        return;
                    }
                }
                else
                {
                    // if not specified, position at the left edge of the screen.
                    left = 0;
                }

                if (args.Length > 3)
                {
                    if (!int.TryParse(args[3], out top))
                    {
                        Console.Error.WriteLine("Failed to parse the top parameter: {0}", args[3]);
                        this.Shutdown(3);
                        return;
                    }
                }
                else
                {
                    // if not specified, position at the top edge of the sceen.
                    top = 0;
                }

                if (args.Length > 4)
                {
                    if (!int.TryParse(args[4], out width))
                    {
                        Console.Error.WriteLine("Failed to parse the width parameter: {0}", args[4]);
                        this.Shutdown(4);
                        return;
                    }
                }
                else
                {
                    // if not specified, use the screen width
                    width = SafeNativeMethods.GetSystemMetrics((int)SystemMetrics.SM_CXSCREEN);

                    if (width == 0)
                    {
                        Console.Error.WriteLine("Failed to get the screen width.");
                        this.Shutdown(7);
                        return;
                    }
                }

                if (args.Length > 5)
                {
                    if (!int.TryParse(args[5], out height))
                    {
                        Console.Error.WriteLine("Failed to parse the top parameter: {0}", args[5]);
                        this.Shutdown(5);
                        return;
                    }
                }
                else
                {
                    // if not specified, use the screen height
                    height = SafeNativeMethods.GetSystemMetrics((int)SystemMetrics.SM_CYSCREEN);

                    if (height == 0)
                    {
                        Console.Error.WriteLine("Failed to get the screen height.");
                        this.Shutdown(8);
                        return;
                    }
                }

                // find the specific process (or processes) identified by the specified name
                var processes = Process.GetProcessesByName(targetProcessName);

                if (!processes.Any())
                {
                    Console.Error.WriteLine("No process with the name \"{0}\" was found.", targetProcessName);
                    this.Shutdown(6);
                    return;
                }

                // target only the first process found
                var targetProcess = processes.First();

                Console.WriteLine("Target Process Id: {0}, Left: {1}, Top: {2}, Width: {3}, Height: {4}",
                    targetProcess.Id, left, top, width, height);

                // remove the window's frame and set its position and size to the specified dimensions
                foreach (var window in ProcessWindow.GetWindowsForProcess(targetProcess))
                {
                    window.RemoveFrame();
                    window.SetPositionAndSize(left, top, width, height);
                }

                // exit without displaying the UI
                this.Shutdown();
            }
        }
    }
}
