﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace MakeWindowFullscreen
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        DispatcherTimer _Timer;
        ObservableCollection<ProcessViewModel> _Items;

        public MainWindow()
        {
            InitializeComponent();

            _Timer = new DispatcherTimer();
            _Timer.Tick += this.Timer_Tick;
            _Timer.Interval = new TimeSpan(0, 0, 1);

            _Items = new ObservableCollection<ProcessViewModel>(GetProcessesSorted().Select(item => new ProcessViewModel(item)));

            this.ProcessList.ItemsSource = _Items;

            _Timer.Start();
        }

        private static IEnumerable<Process> GetProcessesSorted()
        {
            return Process.GetProcesses().OrderBy(item => item.ProcessName).ThenBy(item => item.Id);
        }

        private static int Compare(Process a, ProcessViewModel b)
        {
            int result;

            result = string.Compare(a.ProcessName, b.ProcessName);
            if (result != 0)
                return result;

            result = a.Id.CompareTo(b.ProcessId);
            if (result != 0)
                return result;

            return 0;
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            var processes = GetProcessesSorted().ToList();

            int indexOfItems = 0, indexOfProcesses = 0;

            while (indexOfItems < _Items.Count && indexOfProcesses < processes.Count)
            {
                var process = processes[indexOfProcesses];
                var item = _Items[indexOfItems];

                int comparison = Compare(process, item);

                if (comparison == 0)
                {
                    // same process, move on
                    indexOfItems++;
                    indexOfProcesses++;
                }
                else if (comparison < 0)
                {
                    // add process
                    _Items.Insert(indexOfItems, new ProcessViewModel(process));

                    indexOfItems++;
                    indexOfProcesses++;
                }
                else
                {
                    // remove process
                    _Items.RemoveAt(indexOfItems);

                    indexOfProcesses++;
                }
            }

            // remove processes at the end of the list
            while (indexOfItems < _Items.Count)
            {
                _Items.RemoveAt(indexOfItems);

                indexOfItems++;
            }

            // add processes at the end of the list
            while (indexOfProcesses < processes.Count)
            {
                var process = processes[indexOfProcesses];

                _Items.Insert(indexOfItems, new ProcessViewModel(process));

                indexOfItems++;
                indexOfProcesses++;
            }
        }

        private void MakeFullscreen_Click(object sender, RoutedEventArgs e)
        {
            var screenWidth = (int)SystemParameters.PrimaryScreenWidth;
            var screenHeight = (int)SystemParameters.PrimaryScreenHeight;

            foreach (ProcessViewModel item in this.ProcessList.SelectedItems)
            {
                foreach (var window in ProcessWindow.GetWindowsForProcess(item.ProcessId))
                {
                    window.RemoveFrame();
                    window.SetPositionAndSize(0, 0, screenWidth, screenHeight);
                }
            }
        }
    }
}
